@@ -1,6 +1,6 @@
 package TPath::TypeConstraints;
 {
-  $TPath::TypeConstraints::VERSION = '0.014';
+  $TPath::TypeConstraints::VERSION = '0.015';
 }
 
 # ABSTRACT: assorted type constraints
@@ -8,13 +8,22 @@ package TPath::TypeConstraints;
 use Moose::Util::TypeConstraints;
 use TPath::Grammar qw(%AXES);
 
-class_type $_ for qw(TPath::Attribute TPath::Expression TPath::AttributeTest);
+class_type $_
+  for
+  qw(TPath::Attribute TPath::Expression TPath::AttributeTest TPath::Math TPath::Function);
 
-role_type $_ for qw(TPath::Test::Boolean TPath::Selector TPath::Forester TPath::Predicate);
+role_type $_
+  for
+  qw(TPath::Test::Boolean TPath::Selector TPath::Forester TPath::Predicate TPath::Numifiable);
 
-union 'ATArg', [qw( Num TPath::Attribute Str )];
+union 'ATArg', [qw( Num TPath::Numifiable Str )];
 
-union 'CondArg', [qw(TPath::Attribute TPath::Expression TPath::AttributeTest TPath::Test::Boolean)];
+union 'CondArg',
+  [
+    qw(TPath::Attribute TPath::Expression TPath::AttributeTest TPath::Test::Boolean)
+  ];
+
+union 'MathArg', [qw(TPath::Numifiable Num)];
 
 enum 'Quantifier' => qw( * + ? e );
 
@@ -30,7 +39,7 @@ TPath::TypeConstraints - assorted type constraints
 
 =head1 VERSION
 
-version 0.014
+version 0.015
 
 =head1 AUTHOR
 
